Class: Rugged::Diff
- Inherits:
-
Object
- Object
- Rugged::Diff
- Includes:
- Enumerable
- Defined in:
- lib/rugged/diff.rb,
lib/rugged/diff/hunk.rb,
lib/rugged/diff/line.rb,
lib/rugged/diff/delta.rb,
ext/rugged/rugged_diff.c
Defined Under Namespace
Instance Attribute Summary collapse
-
#owner ⇒ Object
readonly
Returns the value of attribute owner.
Instance Method Summary collapse
- #deltas ⇒ Object
-
#each_delta ⇒ Object
If given a block, yields each delta that is part of the diff.
- #each_line(*args) ⇒ Object
-
#each_patch ⇒ Object
(also: #each)
If given a block, yields each patch that is part of the diff.
-
#find_similar!([options]) ⇒ self
Detects entries in the diff that look like renames or copies (based on the given options) and replaces them with actual rename or copy entries.
-
#merge!(other_diff) ⇒ self
Merges all diff information from
other_diff. -
#patch(*args) ⇒ Object
Return a string containing the diff in patch form.
- #patches ⇒ Object
-
#size ⇒ Integer
Returns the number of deltas/patches in this diff.
-
#sorted_icase? ⇒ Boolean
Returns true when deltas are sorted case insensitively.
-
#stat ⇒ Integer
Returns the number of files/additions/deletions in this diff.
-
#write_patch(*args) ⇒ Object
Write a patch directly to an object which responds to "write".
Instance Attribute Details
#owner ⇒ Object (readonly)
Returns the value of attribute owner.
15 16 17 |
# File 'lib/rugged/diff.rb', line 15 def owner @owner end |
Instance Method Details
#deltas ⇒ Object
21 22 23 |
# File 'lib/rugged/diff.rb', line 21 def deltas each_delta.to_a end |
#each_delta {|delta| ... } ⇒ self #each_delta ⇒ Object
If given a block, yields each delta that is part of the diff. If no block is given, an enumerator will be returned.
This method should be preferred over #each_patch if you're not interested in the actual line-by-line changes of the diff.
491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 |
# File 'ext/rugged/rugged_diff.c', line 491
static VALUE rb_git_diff_each_delta(VALUE self)
{
git_diff *diff;
const git_diff_delta *delta;
size_t d, delta_count;
RETURN_ENUMERATOR(self, 0, 0);
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
delta_count = git_diff_num_deltas(diff);
for (d = 0; d < delta_count; ++d) {
delta = git_diff_get_delta(diff, d);
rb_yield(rugged_diff_delta_new(self, delta));
}
return self;
}
|
#each_line([format = :patch]) {|line| ... } ⇒ self #each_line([format = :patch]) ⇒ Object
525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 |
# File 'ext/rugged/rugged_diff.c', line 525
static VALUE rb_git_diff_each_line(int argc, VALUE *argv, VALUE self)
{
VALUE rb_format;
git_diff *diff;
git_diff_format_t format;
int exception = 0, error;
RETURN_ENUMERATOR(self, argc, argv);
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
if (rb_scan_args(argc, argv, "01", &rb_format) == 1) {
Check_Type(rb_format, T_SYMBOL);
} else {
rb_format = CSTR2SYM("patch");
}
if (SYM2ID(rb_format) == rb_intern("patch")) {
format = GIT_DIFF_FORMAT_PATCH;
} else if (SYM2ID(rb_format) == rb_intern("patch_header")) {
format = GIT_DIFF_FORMAT_PATCH_HEADER;
} else if (SYM2ID(rb_format) == rb_intern("raw")) {
format = GIT_DIFF_FORMAT_RAW;
} else if (SYM2ID(rb_format) == rb_intern("name_only")) {
format = GIT_DIFF_FORMAT_NAME_ONLY;
} else if (SYM2ID(rb_format) == rb_intern("name_status")) {
format = GIT_DIFF_FORMAT_NAME_STATUS;
} else {
rb_raise(rb_eArgError, "unknown :format");
}
error = git_diff_print(diff, format, each_line_cb, &exception);
if (exception)
rb_jump_tag(exception);
rugged_exception_check(error);
return self;
}
|
#each_patch {|patch| ... } ⇒ self #each_patch ⇒ Object Also known as: each
If given a block, yields each patch that is part of the diff. If no block is given, an enumerator will be returned.
457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 |
# File 'ext/rugged/rugged_diff.c', line 457
static VALUE rb_git_diff_each_patch(VALUE self)
{
git_diff *diff;
git_patch *patch;
int error = 0;
size_t d, delta_count;
RETURN_ENUMERATOR(self, 0, 0);
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
delta_count = git_diff_num_deltas(diff);
for (d = 0; d < delta_count; ++d) {
error = git_patch_from_diff(&patch, diff, d);
if (error) break;
rb_yield(rugged_patch_new(self, patch));
}
rugged_exception_check(error);
return self;
}
|
#find_similar!([options]) ⇒ self
Detects entries in the diff that look like renames or copies (based on the given options) and replaces them with actual rename or copy entries.
Additionally, modified files can be broken into add/delete pairs if the
amount of changes are above a specific threshold (see :break_rewrite_threshold).
By default, similarity will be measured without leading whitespace. You
you can use the :dont_ignore_whitespace to disable this.
The following options can be passed in the options Hash:
:rename_threshold ::
An integer specifying the similarity to consider a file renamed (default 50).
:rename_from_rewrite_threshold ::
An integer specifying the similarity of modified to be eligible
rename source (default 50).
:copy_threshold ::
An integer specifying the similarity to consider a file a copy (default 50).
:break_rewrite_threshold ::
An integer specifying the similarity to split modify into delete/add pair (default 60).
:rename_limit ::
An integer specifying the maximum amount of similarity sources to examine
(a la diff's +-l+ option or the +diff.renameLimit+ config) (default 200).
:renames ::
If true, looking for renames will be enabled (+--find-renames+).
:renames_from_rewrites ::
If true, the "old side" of modified files will be considered for renames (+--break-rewrites=N+).
:copies ::
If true, looking for copies will be enabled (+--find-copies+).
:copies_from_unmodified ::
If true, unmodified files will be considered as copy sources (+--find-copies-harder+).
:break_rewrites ::
If true, larger rewrites will be split into delete/add pairs (+--break-rewrites=/M+).
:all ::
If true, enables all finding features.
:ignore_whitespace ::
If true, similarity will be measured with all whitespace ignored.
:dont_ignore_whitespace ::
If true, similarity will be measured without ignoring any whitespace.
368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 |
# File 'ext/rugged/rugged_diff.c', line 368
static VALUE rb_git_diff_find_similar(int argc, VALUE *argv, VALUE self)
{
git_diff *diff;
git_diff_find_options opts = GIT_DIFF_FIND_OPTIONS_INIT;
VALUE rb_options;
int error;
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
rb_scan_args(argc, argv, "00:", &rb_options);
if (!NIL_P(rb_options)) {
VALUE rb_value = rb_hash_aref(rb_options, CSTR2SYM("rename_threshold"));
if (!NIL_P(rb_value)) {
Check_Type(rb_value, T_FIXNUM);
opts.rename_threshold = FIX2INT(rb_value);
}
rb_value = rb_hash_aref(rb_options, CSTR2SYM("rename_from_rewrite_threshold"));
if (!NIL_P(rb_value)) {
Check_Type(rb_value, T_FIXNUM);
opts.rename_from_rewrite_threshold = FIX2INT(rb_value);
}
rb_value = rb_hash_aref(rb_options, CSTR2SYM("copy_threshold"));
if (!NIL_P(rb_value)) {
Check_Type(rb_value, T_FIXNUM);
opts.copy_threshold = FIX2INT(rb_value);
}
rb_value = rb_hash_aref(rb_options, CSTR2SYM("break_rewrite_threshold"));
if (!NIL_P(rb_value)) {
Check_Type(rb_value, T_FIXNUM);
opts.break_rewrite_threshold = FIX2INT(rb_value);
}
rb_value = rb_hash_aref(rb_options, CSTR2SYM("rename_limit"));
if (!NIL_P(rb_value)) {
Check_Type(rb_value, T_FIXNUM);
opts.rename_limit = FIX2INT(rb_value);
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("renames")))) {
opts.flags |= GIT_DIFF_FIND_RENAMES;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("renames_from_rewrites")))) {
opts.flags |= GIT_DIFF_FIND_RENAMES_FROM_REWRITES;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("copies")))) {
opts.flags |= GIT_DIFF_FIND_COPIES;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("copies_from_unmodified")))) {
opts.flags |= GIT_DIFF_FIND_COPIES_FROM_UNMODIFIED;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("break_rewrites")))) {
opts.flags |= GIT_DIFF_FIND_AND_BREAK_REWRITES;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("all")))) {
opts.flags |= GIT_DIFF_FIND_ALL;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("ignore_whitespace")))) {
opts.flags |= GIT_DIFF_FIND_IGNORE_WHITESPACE;
}
if (RTEST(rb_hash_aref(rb_options, CSTR2SYM("dont_ignore_whitespace")))) {
opts.flags |= GIT_DIFF_FIND_DONT_IGNORE_WHITESPACE;
}
}
error = git_diff_find_similar(diff, &opts);
rugged_exception_check(error);
return self;
}
|
#merge!(other_diff) ⇒ self
Merges all diff information from other_diff.
293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'ext/rugged/rugged_diff.c', line 293
static VALUE rb_git_diff_merge(VALUE self, VALUE rb_other)
{
git_diff *diff;
git_diff *other;
int error;
if (!rb_obj_is_kind_of(rb_other, rb_cRuggedDiff))
rb_raise(rb_eTypeError, "A Rugged::Diff instance is required");
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
TypedData_Get_Struct(rb_other, git_diff, &rugged_diff_type, other);
error = git_diff_merge(diff, other);
rugged_exception_check(error);
return self;
}
|
#patch ⇒ Object #patch(: compact) ⇒ Object
Return a string containing the diff in patch form.
221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 |
# File 'ext/rugged/rugged_diff.c', line 221
static VALUE rb_git_diff_patch(int argc, VALUE *argv, VALUE self)
{
git_diff *diff;
VALUE rb_str = rb_str_new(NULL, 0);
VALUE rb_opts;
rb_scan_args(argc, argv, "00:", &rb_opts);
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
if (!NIL_P(rb_opts)) {
if (rb_hash_aref(rb_opts, CSTR2SYM("compact")) == Qtrue)
git_diff_print(diff, GIT_DIFF_FORMAT_NAME_STATUS, diff_print_cb, (void*)rb_str);
else
git_diff_print(diff, GIT_DIFF_FORMAT_PATCH, diff_print_cb, (void*)rb_str);
} else {
git_diff_print(diff, GIT_DIFF_FORMAT_PATCH, diff_print_cb, (void*)rb_str);
}
return rb_str;
}
|
#patches ⇒ Object
17 18 19 |
# File 'lib/rugged/diff.rb', line 17 def patches each_patch.to_a end |
#size ⇒ Integer
Returns the number of deltas/patches in this diff.
570 571 572 573 574 575 576 577 |
# File 'ext/rugged/rugged_diff.c', line 570
static VALUE rb_git_diff_size(VALUE self)
{
git_diff *diff;
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
return INT2FIX(git_diff_num_deltas(diff));
}
|
#sorted_icase? ⇒ Boolean
Returns true when deltas are sorted case insensitively.
647 648 649 650 651 652 |
# File 'ext/rugged/rugged_diff.c', line 647
static VALUE rb_git_diff_sorted_icase_p(VALUE self)
{
git_diff *diff;
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
return git_diff_is_sorted_icase(diff) ? Qtrue : Qfalse;
}
|
#stat ⇒ Integer
Returns the number of files/additions/deletions in this diff.
628 629 630 631 632 633 634 635 636 637 638 639 640 |
# File 'ext/rugged/rugged_diff.c', line 628
static VALUE rb_git_diff_stat(VALUE self)
{
git_diff *diff;
struct diff_stats stats = { 0, 0, 0 };
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
git_diff_foreach(
diff, diff_file_stats_cb, NULL, NULL, diff_line_stats_cb, &stats);
return rb_ary_new3(
3, INT2FIX(stats.files), INT2FIX(stats.adds), INT2FIX(stats.dels));
}
|
#write_patch(io) ⇒ nil #write_patch(io, : compact) ⇒ Object
Write a patch directly to an object which responds to "write".
263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 |
# File 'ext/rugged/rugged_diff.c', line 263
static VALUE rb_git_diff_write_patch(int argc, VALUE *argv, VALUE self)
{
git_diff *diff;
VALUE rb_io, rb_opts;
rb_scan_args(argc, argv, "10:", &rb_io, &rb_opts);
if (!rb_respond_to(rb_io, rb_intern("write")))
rb_raise(rb_eArgError, "Expected io to respond to \"write\"");
TypedData_Get_Struct(self, git_diff, &rugged_diff_type, diff);
if (!NIL_P(rb_opts)) {
if (rb_hash_aref(rb_opts, CSTR2SYM("compact")) == Qtrue)
git_diff_print(diff, GIT_DIFF_FORMAT_NAME_STATUS, diff_write_cb, (void*)rb_io);
else
git_diff_print(diff, GIT_DIFF_FORMAT_PATCH, diff_write_cb, (void*)rb_io);
} else {
git_diff_print(diff, GIT_DIFF_FORMAT_PATCH, diff_write_cb, (void*)rb_io);
}
return Qnil;
}
|